@@ -1,4 +1,13 @@
-/* $Id: menu.h,v 1.5 2001/02/26 23:38:03 edg Exp $ */
+/* $Id: menu.h,v 1.7 2002/07/11 21:18:10 slobasso Exp $ */
+
+#ifndef NEDIT_MENU_H_INCLUDED
+#define NEDIT_MENU_H_INCLUDED
+
+#include "nedit.h"
+
+#include <X11/Intrinsic.h>
+#include <X11/Xlib.h>
+
 #define PERMANENT_MENU_ITEM (XtPointer)1
 #define TEMPORARY_MENU_ITEM (XtPointer)2
 
@@ -12,3 +21,6 @@ void WriteNEditDB(void);
 void ReadNEditDB(void);
 Widget CreateBGMenu(WindowInfo *window);
 void AddBGMenuAction(Widget widget);
+void HidePointerOnKeyedEvent(Widget w, XEvent *event);
+
+#endif /* NEDIT_MENU_H_INCLUDED */
