@@ -1,3 +1,4 @@
+static const char CVSID[] = "$Id: selection.c,v 1.25 2003/05/09 17:43:48 edg Exp $";
 /*******************************************************************************
 *									       *
 * Copyright (C) 1999 Mark Edel						       *
@@ -9,7 +10,7 @@
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
-* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
 * for more details.							       *
 * 									       *
 * You should have received a copy of the GNU General Public License along with *
@@ -22,27 +23,45 @@
 * Written by Mark Edel							       *
 *									       *
 *******************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#include "selection.h"
+#include "textBuf.h"
+#include "text.h"
+#include "nedit.h"
+#include "file.h"
+#include "window.h"
+#include "menu.h"
+#include "server.h"
+#include "../util/DialogF.h"
+#include "../util/fileUtils.h"
+
+#include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
+#include <string.h>
+#include <limits.h>
 #ifdef VMS
 #include "../util/VMSparam.h"
 #else
+#ifndef __MVS__
 #include <sys/param.h>
+#endif
 #endif /*VMS*/
 #if !defined(DONT_HAVE_GLOB) && !defined(USE_MOTIF_GLOB) && !defined(VMS)
 #include <glob.h>
 #endif
+
 #include <Xm/Xm.h>
 #include <X11/Xatom.h>
-#include "../util/DialogF.h"
-#include "../util/fileUtils.h"
-#include "textBuf.h"
-#include "text.h"
-#include "nedit.h"
-#include "selection.h"
-#include "file.h"
-#include "window.h"
-#include "menu.h"
+
+#ifdef HAVE_DEBUG_H
+#include "../debug.h"
+#endif
+
 
 static void gotoCB(Widget widget, WindowInfo *window, Atom *sel,
 	Atom *type, char *value, int *length, int *format);
@@ -64,17 +83,55 @@ static void maintainSelection(selection *sel, int pos, int nInserted,
 static void maintainPosition(int *position, int modPos, int nInserted,
     	int nDeleted);
 
+/*
+** Extract the line and column number from the text string.
+** Set the line and/or column number to -1 if not specified, and return -1 if
+** both line and column numbers are not specified.
+*/
+int StringToLineAndCol(const char *text, int *lineNum, int *column) {
+    char *endptr;
+    long  tempNum;
+    int   textLen;
+
+    /* Get line number */
+    tempNum = strtol( text, &endptr, 10 );
+
+    /* If user didn't specify a line number, set lineNum to -1 */
+    if      ( endptr  == text    ) { *lineNum = -1;      }
+    else if ( tempNum >= INT_MAX ) { *lineNum = INT_MAX; }
+    else if ( tempNum <  0       ) { *lineNum = 0;       }
+    else                           { *lineNum = tempNum; }
+
+    /* Find the next digit */
+    for ( textLen = strlen( endptr ); textLen > 0; endptr++, textLen-- ) {
+        if (isdigit((unsigned char) *endptr ) || *endptr == '-' || *endptr == '+') {
+            break;
+        }
+    }
+
+    /* Get column */
+    if ( *endptr != '\0' ) {
+        tempNum = strtol( endptr, NULL, 10 );
+        if      ( tempNum >= INT_MAX ) { *column = INT_MAX; }
+        else if ( tempNum <  0       ) { *column = 0;       }
+        else                           { *column = tempNum; }
+    }
+    else { *column = -1; }
+
+    return *lineNum == -1 && *column == -1 ? -1 : 0;
+}
+
 void GotoLineNumber(WindowInfo *window)
 {
     char lineNumText[DF_MAX_PROMPT_LENGTH], *params[1];
-    int lineNum, nRead, response;
+    int lineNum, column, response;
     
-    response = DialogF(DF_PROMPT, window->shell, 2, "Goto Line Number:",
-    		       lineNumText, "OK", "Cancel");
+    response = DialogF(DF_PROMPT, window->shell, 2, "Goto Line Number",
+            "Goto Line (and/or Column)  Number:", lineNumText, "OK", "Cancel");
     if (response == 2)
     	return;
-    nRead = sscanf(lineNumText, "%d", &lineNum);
-    if (nRead != 1) {
+
+    if (StringToLineAndCol(lineNumText, &lineNum, &column) == -1) {
     	XBell(TheDisplay, 0);
 	return;
     }
@@ -101,7 +158,7 @@ void OpenSelectedFile(WindowInfo *window, Time time)
 */
 char *GetAnySelection(WindowInfo *window)
 {
-    char waitingMarker[1] = "";
+    static char waitingMarker[1] = "";
     char *selText = waitingMarker;
     XEvent nextEvent;	 
     
@@ -122,7 +179,7 @@ char *GetAnySelection(WindowInfo *window)
     while (selText == waitingMarker) {
 	XtAppNextEvent(XtWidgetToApplicationContext(window->textArea), 
 		&nextEvent);
-	XtDispatchEvent(&nextEvent);
+	ServerDispatchEvent(&nextEvent);
     }
     return selText;
 }
@@ -130,15 +187,16 @@ char *GetAnySelection(WindowInfo *window)
 static void gotoCB(Widget widget, WindowInfo *window, Atom *sel,
     	Atom *type, char *value, int *length, int *format)
 {
-    char lineText[21];
-    int nRead, lineNum;
+     /* two integers and some space in between */
+    char lineText[(TYPE_INT_STR_SIZE(int) * 2) + 5];
+    int rc, lineNum, column, position, curCol;
     
     /* skip if we can't get the selection data, or it's obviously not a number */
     if (*type == XT_CONVERT_FAIL || value == NULL) {
     	XBell(TheDisplay, 0);
 	return;
     }
-    if (*length > 20) {
+    if (((size_t) *length) > sizeof(lineText) - 1) {
     	XBell(TheDisplay, 0);
 	XtFree(value);
 	return;
@@ -150,17 +208,36 @@ static void gotoCB(Widget widget, WindowInfo *window, Atom *sel,
 	XtFree(value);
 	return;
     }
-    strncpy(lineText, value, *length);
-    lineText[*length] = '\0';
+    strncpy(lineText, value, sizeof(lineText));
+    lineText[sizeof(lineText) - 1] = '\0';
     
-    nRead = sscanf(lineText, "%d", &lineNum);
+    rc = StringToLineAndCol(lineText, &lineNum, &column);
     XtFree(value);
-    if (nRead != 1) {
+    if (rc == -1) {
     	XBell(TheDisplay, 0);
 	return;
     }
 
-    SelectNumberedLine(window, lineNum);
+    /* User specified column, but not line number */
+    if ( lineNum == -1 ) {
+        position = TextGetCursorPos(widget);
+        if (TextPosToLineAndCol(widget, position, &lineNum, &curCol) == False) {
+            XBell(TheDisplay, 0);
+            return;
+        }
+    }
+    /* User didn't specify a column */
+    else if ( column == -1 ) {
+        SelectNumberedLine(window, lineNum);
+        return;
+    }
+
+    position = TextLineAndColToPos(widget, lineNum, column );
+    if ( position == -1 ) {
+        XBell(TheDisplay, 0);
+        return;
+    }
+    TextSetCursorPos(widget, position);
 }
 
 static void fileCB(Widget widget, WindowInfo *window, Atom *sel,
@@ -218,7 +295,8 @@ static void fileCB(Widget widget, WindowInfo *window, Atom *sel,
     if ((strchr(nameText, ':') == NULL) && (strlen(nameText) > 1) &&
       	    !((nameText[0] == '[') && (nameText[1] != '-') &&
 	      (nameText[1] != '.'))) {
-	sprintf(filename, "%s%s", window->path, nameText);
+	strcpy(filename, window->path);
+	strcat(filename, nameText);
 	strcpy(nameText, filename);
     }
 #else
@@ -227,7 +305,8 @@ static void fileCB(Widget widget, WindowInfo *window, Atom *sel,
         
     /* If path name is relative, make it refer to current window's directory */
     if (nameText[0] != '/') {
-	sprintf(filename, "%s%s", window->path, nameText);
+	strcpy(filename, window->path);
+	strcat(filename, nameText);
 	strcpy(nameText, filename);
     }
 #endif
@@ -239,29 +318,46 @@ static void fileCB(Widget widget, WindowInfo *window, Atom *sel,
        guranteed to be available, but in practice is there and does work. */
 #if defined(DONT_HAVE_GLOB) || defined(VMS)
     /* Open the file */
-    ParseFilename(nameText, filename, pathname);
-    EditExistingFile(WindowList, filename, pathname, False, NULL, False, NULL);
+    if (ParseFilename(nameText, filename, pathname) != 0) {
+        XBell(TheDisplay, 0);
+	return;
+    }	
+    EditExistingFile(WindowList, filename, pathname, 0, NULL, False, NULL);
 #elif defined(USE_MOTIF_GLOB)
-    { char **nameList = NULL; int i, nFiles = 0, maxFiles = 30;
-      ParseFilename(nameText, filename, pathname);
+    { char **nameList = NULL;
+      int i, nFiles = 0, maxFiles = 30;
+
+      if (ParseFilename(nameText, filename, pathname) != 0) {
+           XBell(TheDisplay, 0);
+	   return;
+      }
       _XmOSGetDirEntries(pathname, filename, XmFILE_ANY_TYPE, False, True,
 	      &nameList, &nFiles, &maxFiles);
       for (i=0; i<nFiles; i++) {
-	  ParseFilename(nameList[i], filename, pathname);
-    	  EditExistingFile(WindowList, filename, pathname, False, NULL, False,
-		  NULL);
+	  if (ParseFilename(nameList[i], filename, pathname) != 0) {
+	      XBell(TheDisplay, 0);
+	  }
+        else {
+    	      EditExistingFile(WindowList, filename, pathname, 0, NULL, 
+	                       False, NULL);
+	  }
       }
-      for (i=0; i<nFiles; i++)
-	  XtFree(nameList[i]);
+      for (i=0; i<nFiles; i++) {
+	   XtFree(nameList[i]);
+	}
       XtFree((char *)nameList);
     }
 #else
-    { glob_t globbuf; int i;
+    { glob_t globbuf;
+      int i;
+
       glob(nameText, GLOB_NOCHECK, NULL, &globbuf);
-      for (i=0; i<globbuf.gl_pathc; i++) {
-	  ParseFilename(globbuf.gl_pathv[i], filename, pathname);
-    	  EditExistingFile(WindowList, filename, pathname, False, NULL, False,
-		  NULL);
+      for (i=0; i<(int)globbuf.gl_pathc; i++) {
+	  if (ParseFilename(globbuf.gl_pathv[i], filename, pathname) != 0)
+	      XBell(TheDisplay, 0);
+	  else
+    	      EditExistingFile(WindowList, filename, pathname, 0, NULL,
+	                       False, NULL);
       }
       globfree(&globbuf);
     }
@@ -290,7 +386,7 @@ static void getAnySelectionCB(Widget widget, char **result, Atom *sel,
 
 void SelectNumberedLine(WindowInfo *window, int lineNum)
 {
-    int i, lineStart, lineEnd;
+    int i, lineStart = 0, lineEnd;
 
     /* count lines to find the start and end positions for the selection */
     if (lineNum < 1)
@@ -302,7 +398,21 @@ void SelectNumberedLine(WindowInfo *window, int lineNum)
     }
     
     /* highlight the line */
-    BufSelect(window->buffer, lineStart, lineEnd+1);
+    if (i>lineNum) {
+	/* Line was found */
+	if (lineEnd < window->buffer->length) {
+	    BufSelect(window->buffer, lineStart, lineEnd+1);
+	} else { 
+	    /* Don't select past the end of the buffer ! */
+	    BufSelect(window->buffer, lineStart, window->buffer->length);
+	}
+    } else {
+	/* Line was not found -> position the selection & cursor at the end 
+	   without making a real selection and beep */
+	lineStart = window->buffer->length;
+	BufSelect(window->buffer, lineStart, lineStart);
+	XBell(TheDisplay, 0);
+    }     
     MakeSelectionVisible(window, window->lastFocus);
     TextSetCursorPos(window->lastFocus, lineStart);
 }
@@ -312,16 +422,15 @@ void MarkDialog(WindowInfo *window)
     char letterText[DF_MAX_PROMPT_LENGTH], *params[1];
     int response;
     
-    response = DialogF(DF_PROMPT, window->shell, 2,
-"Enter a single letter label to use for recalling\n\
-the current selection and cursor position.\n\
-\n\
-(To skip this dialog, use the accelerator key,\n\
-followed immediately by a letter key (a-z))",
-    		       letterText, "OK", "Cancel");
+    response = DialogF(DF_PROMPT, window->shell, 2, "Mark",
+            "Enter a single letter label to use for recalling\n"
+            "the current selection and cursor position.\n\n"
+            "(To skip this dialog, use the accelerator key,\n"
+            "followed immediately by a letter key (a-z))", letterText, "OK",
+            "Cancel");
     if (response == 2)
     	return;
-    if (strlen(letterText) != 1 || !isalpha(letterText[0])) {
+    if (strlen(letterText) != 1 || !isalpha((unsigned char)letterText[0])) {
     	XBell(TheDisplay, 0);
 	return;
     }
@@ -334,16 +443,15 @@ void GotoMarkDialog(WindowInfo *window, int extend)
     char letterText[DF_MAX_PROMPT_LENGTH], *params[2];
     int response;
     
-    response = DialogF(DF_PROMPT, window->shell, 2,
-"Enter the single letter label used to mark\n\
-the selection and/or cursor position.\n\
-\n\
-(To skip this dialog, use the accelerator\n\
-key, followed immediately by the letter)",
-    		       letterText, "OK", "Cancel");
+    response = DialogF(DF_PROMPT, window->shell, 2, "Goto Mark",
+            "Enter the single letter label used to mark\n"
+            "the selection and/or cursor position.\n\n"
+            "(To skip this dialog, use the accelerator\n"
+            "key, followed immediately by the letter)", letterText, "OK",
+            "Cancel");
     if (response == 2)
     	return;
-    if (strlen(letterText) != 1 || !isalpha(letterText[0])) {
+    if (strlen(letterText) != 1 || !isalpha((unsigned char)letterText[0])) {
     	XBell(TheDisplay, 0);
 	return;
     }
@@ -514,10 +622,10 @@ void GotoMark(WindowInfo *window, Widget w, char label, int extendSel)
        cursor position without first using the less pleasing capability
        of the widget itself for bringing the cursor in to view, you have to
        first turn it off, set the position, then turn it back on. */
-    XtVaSetValues(w, textNautoShowInsertPos, False, 0);
+    XtVaSetValues(w, textNautoShowInsertPos, False, NULL);
     TextSetCursorPos(w, cursorPos);
     MakeSelectionVisible(window, window->lastFocus);
-    XtVaSetValues(w, textNautoShowInsertPos, True, 0);
+    XtVaSetValues(w, textNautoShowInsertPos, True, NULL);
 }
 
 /*
