@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: userCmds.c,v 1.17 2001/08/21 14:29:37 tringali Exp $";
+static const char CVSID[] = "$Id: userCmds.c,v 1.32 2003/05/02 19:19:02 edg Exp $";
 /*******************************************************************************
 *									       *
 * userCmds.c -- Nirvana Editor shell and macro command dialogs 		       *
@@ -25,6 +25,27 @@ static const char CVSID[] = "$Id: userCmds.c,v 1.17 2001/08/21 14:29:37 tringali
 * Written by Mark Edel							       *
 *									       *
 *******************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#include "userCmds.h"
+#include "textBuf.h"
+#include "text.h"
+#include "nedit.h"
+#include "preferences.h"
+#include "window.h"
+#include "menu.h"
+#include "shell.h"
+#include "macro.h"
+#include "file.h"
+#include "interpret.h"
+#include "parse.h"
+#include "../util/DialogF.h"
+#include "../util/misc.h"
+#include "../util/managedList.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -49,22 +70,10 @@ static const char CVSID[] = "$Id: userCmds.c,v 1.17 2001/08/21 14:29:37 tringali
 #include <Xm/RowColumn.h>
 #include <Xm/CascadeB.h>
 
-#include "../util/DialogF.h"
-#include "../util/misc.h"
-#include "../util/managedList.h"
-#include "textBuf.h"
-#include "text.h"
-#include "nedit.h"
-#include "preferences.h"
-#include "window.h"
-#include "menu.h"
-#include "shell.h"
-#include "macro.h"
-#include "file.h"
-#include "rbTree.h"
-#include "interpret.h"
-#include "parse.h"
-#include "userCmds.h"
+#ifdef HAVE_DEBUG_H
+#include "../debug.h"
+#endif
+
 
 /* max number of user programmable menu commands allowed per each of the
    macro, shell, and bacground menus */
@@ -138,9 +147,9 @@ static void dimSelDepItemsInMenu(Widget menuPane, menuItemRec **menuList,
 	int nMenuItems, int sensitive);
 static void updateMenus(int menuType);
 static Widget findInMenuTree(menuTreeItem *menuTree, int nTreeEntries,
-	char *hierName);
-static char *copySubstring(char *string, int length);
-static char *findStripLanguageMode(char *menuItemName, int languageMode,
+                             const char *hierName);
+static char *copySubstring(const char *string, int length);
+static char *findStripLanguageMode(const char *menuItemName, int languageMode,
 	int *isDefaultLM);
 static Widget createUserMenuItem(Widget menuPane, char *name, menuItemRec *f,
 	int index, XtCallbackProc cbRtn, XtPointer cbArg);
@@ -182,7 +191,7 @@ static void generateAcceleratorString(char *text, unsigned int modifiers,
 	KeySym keysym);
 static void genAccelEventName(char *text, unsigned int modifiers,
 	KeySym keysym);
-static int parseAcceleratorString(char *string, unsigned int *modifiers,
+static int parseAcceleratorString(const char *string, unsigned int *modifiers,
 	KeySym *keysym);
 static int parseError(const char *message);
 static char *copyMacroToEnd(char **inPtr);
@@ -220,8 +229,8 @@ void EditShellMenu(WindowInfo *window)
     
     ac = 0;
     XtSetArg(args[ac], XmNdeleteResponse, XmDO_NOTHING); ac++;
-    XtSetArg(args[ac], XmNiconName, "Shell Commands"); ac++;
-    XtSetArg(args[ac], XmNtitle, "Shell Commands"); ac++;
+    XtSetArg(args[ac], XmNiconName, "NEdit Shell Menu"); ac++;
+    XtSetArg(args[ac], XmNtitle, "Shell Menu"); ac++;
     ucd->dlogShell = CreateShellWithBestVis(APP_NAME, APP_CLASS,
 	    applicationShellWidgetClass, TheDisplay, args, ac);
     AddSmallIcon(ucd->dlogShell);
@@ -499,7 +508,7 @@ Select \"New\" to add a new command to the menu."),
     	    XmNleftPosition, LEFT_MARGIN_POS, NULL);
     XmStringFree(s1);
     XtVaCreateManagedWidget("cmdLabel", xmLabelGadgetClass, form,
-    	    XmNlabelString, s1=MKSTRING("(% expands to current filename)"),
+    	    XmNlabelString, s1=MKSTRING("(% expands to current filename, # to line number)"),
     	    XmNalignment, XmALIGNMENT_END,
     	    XmNmarginTop, 5,
     	    XmNtopAttachment, XmATTACH_POSITION,
@@ -558,6 +567,7 @@ Select \"New\" to add a new command to the menu."),
     XtSetArg(args[ac], XmNbottomWidget, okBtn); ac++;
     XtSetArg(args[ac], XmNbottomOffset, 5); ac++;
     ucd->cmdTextW = XmCreateScrolledText(form, "name", args, ac);
+    AddMouseWheelSupport(ucd->cmdTextW);
     XtManageChild(ucd->cmdTextW);
     MakeSingleLineTextW(ucd->cmdTextW);
     RemapDeleteKey(ucd->cmdTextW);
@@ -860,6 +870,7 @@ Select \"New\" to add a new command to the menu."),
     XtSetArg(args[ac], XmNbottomWidget, okBtn); ac++;
     XtSetArg(args[ac], XmNbottomOffset, 5); ac++;
     ucd->cmdTextW = XmCreateScrolledText(form, "name", args, ac);
+    AddMouseWheelSupport(ucd->cmdTextW);
     XtManageChild(ucd->cmdTextW);
     RemapDeleteKey(ucd->cmdTextW);
     XtVaSetValues(cmdLabel, XmNuserData, ucd->cmdTextW, NULL); /* for mnemonic */
@@ -946,7 +957,7 @@ static void dimSelDepItemsInMenu(Widget menuPane, menuItemRec **menuList,
     Cardinal nItems;
     
     XtVaGetValues(menuPane, XmNchildren, &items, XmNnumChildren, &nItems, NULL);
-    for (n=0; n<nItems; n++) {
+    for (n=0; n<(int)nItems; n++) {
 	XtVaGetValues(items[n], XmNuserData, &userData, NULL);
     	if (userData !=  (XtPointer)PERMANENT_MENU_ITEM) {
     	    if (XtClass(items[n]) == xmCascadeButtonWidgetClass) {
@@ -1221,7 +1232,7 @@ static void updateMenu(WindowInfo *window, int menuType)
 ** Find the widget corresponding to a hierarchical menu name (a>b>c...)
 */
 static Widget findInMenuTree(menuTreeItem *menuTree, int nTreeEntries,
-	char *hierName)
+	const char *hierName)
 {
     int i;
     
@@ -1231,7 +1242,7 @@ static Widget findInMenuTree(menuTreeItem *menuTree, int nTreeEntries,
     return NULL;
 }
 
-static char *copySubstring(char *string, int length)
+static char *copySubstring(const char *string, int length)
 {
     char *retStr = XtMalloc(length + 1);
     
@@ -1248,7 +1259,7 @@ static char *copySubstring(char *string, int length)
 ** or an allocated copy of menuItemName stripped of @ parts.  If the
 ** language name is "*", sets isDefaultLM to true.
 */
-static char *findStripLanguageMode(char *menuItemName, int languageMode,
+static char *findStripLanguageMode(const char *menuItemName, int languageMode,
 	int *isDefaultLM)
 {
     char *atPtr, *firstAtPtr, *endPtr;
@@ -1257,7 +1268,9 @@ static char *findStripLanguageMode(char *menuItemName, int languageMode,
     atPtr = firstAtPtr = strchr(menuItemName, '@');
     *isDefaultLM = False;
     if (atPtr == NULL)
-	return CopyAllocatedString(menuItemName);
+    {
+        return XtNewString(menuItemName);
+    }
     if (!strcmp(atPtr+1, "*")) {
 	/* only language is "*": this is for all but language specific macros */
 	*isDefaultLM = True;
@@ -1266,9 +1279,9 @@ static char *findStripLanguageMode(char *menuItemName, int languageMode,
     if (languageMode == PLAIN_LANGUAGE_MODE)
 	return NULL;
     for (;;) {
-	for(endPtr=atPtr+1; isalnum(*endPtr) || *endPtr=='_' || *endPtr=='-' || 
-		*endPtr==' ' || *endPtr=='+' || *endPtr=='$' || *endPtr=='#';
-		endPtr++);
+	for(endPtr=atPtr+1; isalnum((unsigned char)*endPtr) || *endPtr=='_' || 
+	      	*endPtr=='-' || *endPtr==' ' || *endPtr=='+' || *endPtr=='$' || 
+	      	*endPtr=='#'; endPtr++);
 	lmNameLen = endPtr-atPtr-1;
 	if (!strncmp(LanguageModeName(languageMode), atPtr+1, lmNameLen) &&
 		LanguageModeName(languageMode)[lmNameLen] == '\0')
@@ -1338,7 +1351,7 @@ static void removeMenuItems(Widget menuPane)
     memcpy(items, itemList, sizeof(Widget) * nItems);
     
     /* Delete all of the widgets not marked as PERMANENT_MENU_ITEM */
-    for (n=0; n<nItems; n++) {
+    for (n=0; n<(int)nItems; n++) {
 	XtVaGetValues(items[n], XmNuserData, &userData, NULL);
     	if (userData !=  (XtPointer)PERMANENT_MENU_ITEM) {
     	    if (XtClass(items[n]) == xmCascadeButtonWidgetClass) {
@@ -1410,8 +1423,10 @@ static void checkCB(Widget w, XtPointer clientData, XtPointer callData)
     userCmdDialog *ucd = (userCmdDialog *)clientData;
     
     if (checkMacro(ucd))
-	DialogF(DF_INF, ucd->dlogShell, 1, "Macro compiled without error",
-	    "Dismiss");
+    {
+        DialogF(DF_INF, ucd->dlogShell, 1, "Macro",
+                "Macro compiled without error", "Dismiss");
+    }
 }
 
 static int checkMacro(userCmdDialog *ucd)
@@ -1612,8 +1627,10 @@ static void macroMenuCB(Widget w, WindowInfo *window, XtPointer callData)
        repeat any operation, and to embed macros within eachother at any
        level, however, a call here with a macro running means that THE USER
        is explicitly invoking another macro via the menu or an accelerator. */
-    if (window->macroCmdData != NULL)
+    if (window->macroCmdData != NULL) {
+	XBell(TheDisplay, 0);
 	return;
+    }
     
     /* get the index of the macro command and verify that it's in range */
     XtVaGetValues(w, XmNuserData, &userData, NULL);
@@ -1632,6 +1649,12 @@ static void bgMenuCB(Widget w, WindowInfo *window, XtPointer callData)
     int index;
     char *params[1];
 
+    /* Same remark as for macro menu commands (see above). */
+    if (window->macroCmdData != NULL) {
+	XBell(TheDisplay, 0);
+	return;
+    }
+    
     /* get the index of the macro command and verify that it's in range */
     XtVaGetValues(w, XmNuserData, &userData, NULL);
     index = (int)userData - 10;
@@ -1709,38 +1732,52 @@ static menuItemRec *readDialogFields(userCmdDialog *ucd, int silent)
     menuItemRec *f;
 
     nameText = XmTextGetString(ucd->nameTextW);
-    if (*nameText == '\0') {
-    	if (!silent) {
-    	    DialogF(DF_WARN, ucd->dlogShell, 1,
-    		    "Please specify a name\nfor the menu item", "Dismiss");
-    	    XmProcessTraversal(ucd->nameTextW, XmTRAVERSE_CURRENT);
-    	}
-    	XtFree(nameText);
-    	return NULL;
+    if (*nameText == '\0')
+    {
+        if (!silent)
+        {
+            DialogF(DF_WARN, ucd->dlogShell, 1, "Menu Entry",
+                    "Please specify a name\nfor the menu item", "Dismiss");
+            XmProcessTraversal(ucd->nameTextW, XmTRAVERSE_CURRENT);
+        }
+        XtFree(nameText);
+        return NULL;
     }
-    if (strchr(nameText, ':')) {
-    	if (!silent) {
-    	    DialogF(DF_WARN, ucd->dlogShell, 1,
-    		    "Menu item names may not\ncontain colon (:) characters",
-    		    "Dismiss");
-    	    XmProcessTraversal(ucd->nameTextW, XmTRAVERSE_CURRENT);
-    	}
-    	XtFree(nameText);
-    	return NULL;
+
+    if (strchr(nameText, ':'))
+    {
+        if (!silent)
+        {
+            DialogF(DF_WARN, ucd->dlogShell, 1, "Menu Entry",
+                    "Menu item names may not\ncontain colon (:) characters",
+                    "Dismiss");
+            XmProcessTraversal(ucd->nameTextW, XmTRAVERSE_CURRENT);
+        }
+        XtFree(nameText);
+        return NULL;
     }
+
     cmdText = XmTextGetString(ucd->cmdTextW);
-    if (cmdText == NULL || *cmdText == '\0') {
-    	if (!silent) {
-    	    DialogF(DF_WARN, ucd->dlogShell, 1, "Please specify %s to execute",
-    	    	    "Dismiss", ucd->dialogType == SHELL_CMDS ? "shell command" :
-    	    	    "macro command(s)");
-    	    XmProcessTraversal(ucd->cmdTextW, XmTRAVERSE_CURRENT);
-    	}
-    	XtFree(nameText);
-    	if (cmdText!=NULL)
-    	    XtFree(cmdText);
-    	return NULL;
+    if (cmdText == NULL || *cmdText == '\0')
+    {
+        if (!silent)
+        {
+            DialogF(DF_WARN, ucd->dlogShell, 1, "Command to Execute",
+                    "Please specify %s to execute", "Dismiss",
+                    ucd->dialogType == SHELL_CMDS
+                            ? "shell command"
+                            : "macro command(s)");
+            XmProcessTraversal(ucd->cmdTextW, XmTRAVERSE_CURRENT);
+        }
+        XtFree(nameText);
+
+        if (cmdText!=NULL)
+        {
+            XtFree(cmdText);
+        }
+        return NULL;
     }
+
     if (ucd->dialogType == MACRO_CMDS || ucd->dialogType == BG_MENU_CMDS) {
     	addTerminatingNewline(&cmdText);
 	if (!checkMacroText(cmdText, silent ? NULL : ucd->dlogShell,
@@ -1838,13 +1875,16 @@ static void *getDialogDataCB(void *oldItem, int explicitRequest, int *abort,
     	return (void *)currentFields;
 
     /* If user might not be expecting fields to be read, give more warning */
-    if (!explicitRequest) {
-	if (DialogF(DF_WARN, ucd->dlogShell, 2,
-    		"Discard incomplete entry\nfor current menu item?", "Keep",
-    		"Discard") == 2) {
-     	    return oldItem == NULL ? NULL :
-     	    	    (void *)copyMenuItemRec((menuItemRec *)oldItem);
-	}
+    if (!explicitRequest)
+    {
+        if (DialogF(DF_WARN, ucd->dlogShell, 2, "Discard Entry",
+                "Discard incomplete entry\nfor current menu item?", "Keep",
+                "Discard") == 2)
+        {
+            return oldItem == NULL
+                    ? NULL
+                    : (void *)copyMenuItemRec((menuItemRec *)oldItem);
+        }
     }
     
     /* Do readDialogFields again without "silent" mode to display warning(s) */
@@ -1916,9 +1956,9 @@ static char *writeMenuItemString(menuItemRec **menuItems, int nItems,
     for (i=0; i<nItems; i++) {
     	f = menuItems[i];
     	generateAcceleratorString(accStr, f->modifiers, f->keysym);
-    	length += strlen(f->name);
+    	length += strlen(f->name) * 2; /* allow for \n & \\ expansions */
     	length += strlen(accStr);
-    	length += strlen(f->cmd) * 4;	/* allow for \n & \\ expansions */
+    	length += strlen(f->cmd) * 6;	/* allow for \n & \\ expansions */
     	length += 21;			/* number of characters added below */
     }
     length++;				/* terminating null */
@@ -1932,8 +1972,17 @@ static char *writeMenuItemString(menuItemRec **menuItems, int nItems,
     	f = menuItems[i];
     	generateAcceleratorString(accStr, f->modifiers, f->keysym);
     	*outPtr++ = '\t';
-    	strcpy(outPtr, f->name);
-    	outPtr += strlen(f->name);
+    	for (c=f->name; *c!='\0'; ++c) { /* Copy the command name */
+    	    if (*c == '\\') {            /* changing backslashes to \\ */
+    	    	*outPtr++ = '\\';
+ 	    	*outPtr++ = '\\';
+ 	    } else if (*c == '\n') { /* changing newlines to \n */
+ 	    	*outPtr++ = '\\';
+ 	    	*outPtr++ = 'n';
+ 	    } else {
+ 	    	*outPtr++ = *c;
+            }
+    	}
     	*outPtr++ = ':';
     	strcpy(outPtr, accStr);
     	outPtr += strlen(accStr);
@@ -2157,32 +2206,34 @@ static int parseError(const char *message)
 static void generateAcceleratorString(char *text, unsigned int modifiers,
 	KeySym keysym)
 {
-    char *shiftStr = "", *lockStr = "", *ctrlStr = "", *altStr = "";
+    char *shiftStr = "", *ctrlStr = "", *altStr = "";
     char *mod2Str  = "", *mod3Str = "", *mod4Str = "", *mod5Str = "";
     char keyName[20];
+    Modifiers numLockMask = GetNumLockModMask(TheDisplay);
 
     /* if there's no accelerator, generate an empty string */
     if (keysym == NoSymbol) {
     	*text = '\0';
     	return;
     }
+    
 
-    /* translate the modifiers into strings */
+    /* Translate the modifiers into strings.
+       Lock and NumLock are always ignored (see util/misc.c),
+       so we don't display them either. */
     if (modifiers & ShiftMask)
     	shiftStr = "Shift+";
-    if (modifiers & LockMask)
-    	lockStr = "Lock+";
     if (modifiers & ControlMask)
     	ctrlStr = "Ctrl+";
     if (modifiers & Mod1Mask)
     	altStr = "Alt+";
-    if (modifiers & Mod2Mask)
+    if ((modifiers & Mod2Mask) && (Mod2Mask != numLockMask))
         mod2Str = "Mod2+";
-    if (modifiers & Mod3Mask)
+    if ((modifiers & Mod3Mask) && (Mod3Mask != numLockMask))
         mod3Str = "Mod3+";
-    if (modifiers & Mod4Mask)
+    if ((modifiers & Mod4Mask) && (Mod4Mask != numLockMask))
         mod4Str = "Mod4+";
-    if (modifiers & Mod5Mask)
+    if ((modifiers & Mod5Mask) && (Mod5Mask != numLockMask))
         mod5Str = "Mod5+";
     
     /* for a consistent look to the accelerator names in the menus,
@@ -2191,7 +2242,7 @@ static void generateAcceleratorString(char *text, unsigned int modifiers,
     *keyName = toupper(*keyName);
     
     /* concatenate the strings together */
-    sprintf(text, "%s%s%s%s%s%s%s%s%s", shiftStr, lockStr, ctrlStr, altStr, 
+    sprintf(text, "%s%s%s%s%s%s%s%s", shiftStr, ctrlStr, altStr, 
             mod2Str, mod3Str, mod4Str, mod5Str, keyName);
 }
 
@@ -2241,7 +2292,7 @@ static void genAccelEventName(char *text, unsigned int modifiers,
 ** and a KeySym code.  Returns false if string can't be read
 ** ... does not handle whitespace in string (look at scanf)
 */
-static int parseAcceleratorString(char *string, unsigned int *modifiers,
+static int parseAcceleratorString(const char *string, unsigned int *modifiers,
 	KeySym *keysym)
 {
     int i, nFields, inputLength = strlen(string);
