@@ -1,4 +1,8 @@
-/* $Id: vmsUtils.h,v 1.3 2001/02/26 23:38:03 edg Exp $ */
+/* $Id: vmsUtils.h,v 1.5 2002/07/11 21:18:09 slobasso Exp $ */
+
+#ifndef NEDIT_VMSUTILS_H_INCLUDED
+#define NEDIT_VMSUTILS_H_INCLUDED
+
 #ifdef VMS
 #ifndef __DESCRIP_LOADED
 #include descrip
@@ -20,3 +24,5 @@ void VMSFileScanDone(void);
 int ProcAlive(const unsigned int pID);
 
 #endif /*VMS*/
+
+#endif /* NEDIT_VMSUTILS_H_INCLUDED */
